from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import DirectoryView, addDirectoryViews, createDirectoryView
from Products.CMFCore.utils import getToolByName, minimalpath
from Products.CMFRating import rating_globals
from Products.ExternalMethod import ExternalMethod
from App.Common import package_home
from os import path, listdir, stat
from cStringIO import StringIO
import string

out = StringIO()

type_classes = ()

def registerType(self, fti):
    """
        Register type defined in fti in the types tool
    """
    typestool = getToolByName(self, 'portal_types')
    if fti['id'] not in typestool.objectIds():
        cfm = apply(ContentFactoryMetadata, (), fti)
        typestool._setObject(fti['id'], cfm)
        out.write('Registered %s with the types tool\n' % fti['id'])
    else:
        out.write('Object "%s" already existed in the types tool\n' % (fti['id']))

def unregisterType(self, type):
    """
        Un-Register type in the types tool
    """
    typestool = getToolByName(self, 'portal_types')
    if type in typestool.objectIds():
        typestool._delObject(type)
        out.write('Un-Registered %s with the types tool\n' % type)
    else:
        out.write('Object "%s" did not exist in the types tool\n' % type)

def removeSkin(self, skinname):
    """
    Remove a skin from portal_skins
    """
    skinstool = getToolByName(self, 'portal_skins')
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if skinname in path:
            path.remove(skinname)
            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Removed %s from %s skin\n" % (skinname, skin))

    if skinname in skinstool.objectIds():
        skinstool._delObject(skinname)
        out.write('Removed %s from the skinstool\n' % skinname)
        

def install(self):
    """
    Register CMFRating classes with the necessary tools
    """

    out.write('CMF Rating Product Installation on %s\n' % self.id)
    out.write('======================\n\n')

    typestool = getToolByName(self, 'portal_types')
    actionstool = getToolByName(self, 'portal_actions')
    skinstool = getToolByName(self, 'portal_skins')
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    portalcatalog = getToolByName(self, 'portal_catalog')
    wftool = getToolByName(self, 'portal_workflow')

    for c in type_classes:
        t = c.factory_type_information
        if type(t) == type(()):
            for fti in t:
                registerType(self, fti)
        else:
            registerType(self, t)

    # Create and register tools
    tools = {'portal_rating':'Rating Tool'}

    for tool in tools.keys():
        if not tool in portalroot.objectIds():
            portalroot.manage_addProduct['CMFRating'].manage_addTool(tools[tool], None)
            out.write('Created %s\n' % tools[tool])
        else:
            out.write('%s already exists\n' % tools[tool])

    """
    ap = actionstool.listActionProviders()
    for tool in tools.keys():
        if (type(ap) is not type([]) and type(ap) is not type(())) or not tool in ap:
            actionstool.addActionProvider(tool)
            out.write('Added %s as action provider\n' % tools[tool])
        else:
            out.write('%s already existed as action provider\n' % tools[tool])
            
    wftool.setChainForPortalTypes(('Portal Template',), ())
    """

    if not 'Rating' in portalcatalog.Indexes.objectIds():
        portalcatalog.addIndex('Rating', 'FieldIndex')
        out.write('Added "Rating" to the portal catalog\n')
    else:
        out.write('"Rating" already existed in the portal catalog\n')
        
    if not 'CreatorRating' in portalcatalog.Indexes.objectIds():
        portalcatalog.addIndex('CreatorRating', 'FieldIndex')
        out.write('Added "CreatorRating" to the portal catalog\n')
    else:
        out.write('"CreatorRating" already existed in the portal catalog\n')

    # Setup the skins

    skin_name = 'rating'

    if skin_name not in skinstool.objectIds():
        # We need to add Filesystem Directory Views for any directories
        # in our skins/ directory.  These directories should already be
        # configured.
        fp = minimalpath("%s/zpt_skins" % package_home(rating_globals))
        createDirectoryView(skinstool, fp, skin_name)
        out.write("Added %s directory view to portal_skins\n" % fp)
    else:
        out.write("%s directory view already existed in portal_skins\n" % skin_name)


    # Now we need to go through the skin configurations and insert
    # 'prosa_skins' into the configurations.  Preferably, this
    # should be right before where 'content' is placed.  Otherwise, we
    # append it to the end.
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if skin_name not in path:
            try: path.insert(path.index('portal_layout_skins') + 1, skin_name)
            except ValueError:
                try:
                    path.insert(path.index('custom') + 1, skin_name)
                except ValueError:
                    path.append(skin_name)

            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Added '%s' to %s 'skin'\n" % (skin_name, skin))
        else:
            out.write("Skipping %s skin, '%s' is already set up\n" % (skin, skin_name))

    # Add portlet
    macros = 'here/portlet_rating/macros/portlet'
    slots  = self.getProperty('right_slots')
    if self.hasProperty('right_slots') and macros not in slots:
        rs = (macros,) + slots
        self.manage_changeProperties({'right_slots':rs})
        out.write("Added 'portlet_rating' to right_slots\n")

    return out.getvalue()

def uninstall(self):
    """
    Remove skins, types and special objects
    """
    out.write('CMF Rating Un-Installation\n')
    out.write('===========================\n\n')
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    for c in type_classes:
        t = c.factory_type_information
        if type(t) == type(()):
            for fti in t:
                unregisterType(self, fti['id'])
        else:
            unregisterType(self, t['id'])

    removeSkin(self, 'portal_rating_skins')

    return out.getvalue()
