##############################################################################
#    Copyright (C) 2002-2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, sys, urllib, zLOG
from Globals import InitializeClass, Persistent, package_home
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName, UniqueObject
import Products.CMFCore.CMFCorePermissions
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.PageTemplates import PageTemplateFile
from zExceptions import Unauthorized

# Import permission names
from Products.CMFCore import CMFCorePermissions

_zptdir = os.path.join( package_home( globals() ), 'www' )

def addRatingTool(self, id, REQUEST=None):
    """
        Create an empty RatingTool
    """
    tool_object = RatingTool(id)
    self._setObject(id, tool_object)

class RatingTool(UniqueObject, SimpleItem):
    """
        A simple tool for adding the ability to rate portal objects
    """

    meta_type='Rating Tool'
    id = 'portal_rating'
    ratings = {}
    change_ratings = 1
    _allowed_types = []
    _max_rate = 5

    security = ClassSecurityInfo()
    security.declareObjectPublic()

    manage_options = (
                       (
                         {'label':'Ratings','action':'manage_ratings'}
                        ,{ 'label':'Configuration','action':'manage_configuration' }
                      ,) +
                        SimpleItem.manage_options +
                       (
                         {'label':'Overview','action':'manage_overview' },
                       )
                     )

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = PageTemplateFile.PageTemplateFile( 'explainRatingTool', _zptdir )

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_ratings')
    manage_ratings = PageTemplateFile.PageTemplateFile('manageRatings', _zptdir )

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_configuration')
    manage_configuration = PageTemplateFile.PageTemplateFile('manageRatingConfiguration', _zptdir )

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_purgeRatings')
    def manage_purgeRatings(self, REQUEST=None):
        """
        Purge ratings for objects that no longer exists
        """
        purged = 0
        ratings = self.ratings
        for path in ratings.keys():
            try:
                obj = self.restrictedTraverse(path)
            except KeyError:
                del ratings[path]
                purged = purged+1
            except:
                pass

        self.ratings = ratings
        self._p_changed=1
        if REQUEST is not None:
            return self.manage_ratings(
                self, REQUEST, manage_tabs_message='%d rating(s) purged.' % purged)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_delRatings')
    def manage_delRatings(self, ratings=None, REQUEST=None):
        """
        Delete ratings
        """
        deleted = 0
        if ratings:
            own_ratings = self.ratings
            if type(ratings) == type([]):
                deleted = len(ratings)
                for path in ratings:
                    del own_ratings[path]
            else:
                deleted = 1
                del own_ratings[ratings]

            msg = '%d rating(s) deleted.' % deleted
            self.ratings = own_ratings
            self._p_changed=1
        else:
            msg = 'Please select one or more ratings to delete.'
        if REQUEST is not None:
            return self.manage_ratings(
                self, REQUEST, manage_tabs_message=msg)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_editConfiguration')
    def manage_editConfiguration(self, change_ratings, max_rate, allowed_types, adjust_ratings=None, REQUEST=None):
        """
        Save the tool settings
        """
        self.change_ratings=int(change_ratings)
        if self._max_rate != max_rate and adjust_ratings:
            self._adjustRatings(max_rate)
        self._max_rate=int(max_rate)
        self._allowed_types = allowed_types
        if REQUEST is not None:
            return self.manage_configuration(
                self, REQUEST, manage_tabs_message='Saved.')

    def _adjustRatings(self, new_max):
        """
        Adjust the current ratings according to a new max rating
        """
        ratings = self.ratings
        for path in ratings.keys():
            rating = ratings[path]
            for username in rating.keys():
                rate = ratings[path][username]
                ratings[path][username] = (float(rate)/float(self._max_rate))*new_max
        self.ratings = ratings
        self._p_changed = 1

    security.declareProtected(CMFCorePermissions.View, 'setRating')
    def setRating(self, content=None, rate=None, REQUEST=None):
        """
        Rate portal content
        """
        if None in [rate, content]: return
        if not self.isRateable(content):
            raise Unauthorized, '%s is not rateable' % content.Type()

        try:
            path = getToolByName(self, 'portal_url').getRelativeContentURL(content)
            #path = content.absolute_url(relative=1)
        except:
            return

        mtool = getToolByName(self, 'portal_membership')
        if mtool.isAnonymousUser():
            raise Unauthorized, 'Anonymous users are not allowed to rate content'
        uname = mtool.getAuthenticatedMember().getUserName()

        ratings = self.ratings
        if ratings.has_key(path):
            rating = ratings[path]
        else:
            ratings[path] = {}
            rating = ratings[path]

        if rating.has_key(uname):
            if not self.change_ratings:
                raise Unauthorized, 'You have already rated %s' % content.absolute_url()
        rating[uname] = int(rate)
        ratings[path] = rating
        self.ratings = ratings
        self._p_changed = 1
        content.reindexObject()

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect("%s?portal_status_message=%s" % (REQUEST.get('HTTP_REFERER'), "Rating+saved"))

    security.declarePublic('getUserRating')
    def getUserRating( self, content, username=None ):
        """
            Get rating for content. If username is None Authenticated User is queried
        """
        if username is None:
            mtool = getToolByName(self, 'portal_membership')
            if mtool.isAnonymousUser():
                return 0
                raise Unauthorized, 'Anonymous users are not allowed to rate content'
            username = mtool.getAuthenticatedMember().getUserName()

        try:
            path = getToolByName(self, 'portal_url').getRelativeContentURL(content)
            rate = self.ratings[path][username]
        except:
            #zLOG.LOG('Rating Tool.getUserRating:', zLOG.INFO, 'No rate entry for %s at %s. %s - %s' % (username, content.absolute_url(relative=1), sys.exc_info()[0], sys.exc_info()[1]))
            rate = 0
        return rate

    security.declarePublic('getTotalRating')
    def getTotalRating( self, content ):
        """
            Get total rating for content.
        """
        try:
            path = getToolByName(self, 'portal_url').getRelativeContentURL(content)
            rating = self.ratings[path]
        except:
            return 0

        rate = 0
        for username in rating.keys():
            rate = rate + rating[username]
        return rate

    security.declarePublic('getNumRating')
    def getNumRating( self, content ):
        """
            Get number of ratings for content.
        """
        try:
            path = getToolByName(self, 'portal_url').getRelativeContentURL(content)
            rating = self.ratings[path]
        except:
            return 0

        return len(rating)

    security.declarePublic('getMaxRating')
    def getMaxRating( self, content ):
        """
            Get max rating for content.
        """
        try:
            path = getToolByName(self, 'portal_url').getRelativeContentURL(content)
            rating = self.ratings[path]
        except:
            return 0

        return len(rating)*self.maxRate()

    security.declarePublic('getPercentRating')
    def getPercentRating( self, content ):
        """
            Get percent rating for content.
        """
        try:
            path = getToolByName(self, 'portal_url').getRelativeContentURL(content)
            rating = self.ratings[path]
        except:
            return 0

        return (float(self.getTotalRating(content))/float(self.getMaxRating(content)))*100

    security.declarePublic('getTotalRating')
    def getRatings( self ):
        """
            Get all ratings as a list of {'path','rate','num','max','percent'} dictionaries.
        """
        rates = []
        for path in self.ratings.keys():
            rate = 0
            num = 0
            rating = self.ratings[path]
            for username in rating.keys():
                rate = rate + rating[username]
                num = num + 1
            max = self.maxRate()*num
            percent = float(float(rate)/float(max))*100
            rates.append({'path':path,'rate':rate,'num':num,'max':max,'percent':percent})

        return rates

    security.declarePublic('isRatingAllowed')
    def isRatingAllowed( self, content, username=None ):
        """
            Return true if rating is allowed for content
        """
        if username is None:
            mtool = getToolByName(self, 'portal_membership')
            if mtool.isAnonymousUser():
                return 0
            username = mtool.getAuthenticatedMember().getUserName()

        try:
            path = getToolByName(self, 'portal_url').getRelativeContentURL(content)
            rate = self.ratings[path][username]
            #zLOG.LOG('Rating Tool:', zLOG.INFO, 'Found rate entry for %s at %s. Change rating: %d' % (username, path, self.change_ratings))
            return self.change_ratings
        except:
            #zLOG.LOG('Rating Tool:', zLOG.INFO, 'No rate entry for %s at %s. %s - %s' % (username, content.absolute_url(relative=1), sys.exc_info()[0], sys.exc_info()[1]))
            return 1

    security.declarePublic('isRateable')
    def isRateable( self, content ):
        """
            Return true if content is rateable
        """
        return self.isTypeRateable(content.Type())

    security.declarePublic('isTypeRateable')
    def isTypeRateable( self, type ):
        """
            Return true if type is rateable
        """
        return type in self._allowed_types

    security.declarePublic('maxRate')
    def maxRate( self ):
        """
            Return the highest rate a user can give
        """
        return self._max_rate

InitializeClass(RatingTool)